// myimage.h
#ifndef MYIMAGE_H_INCLUDED
#define MYIMAGE_H_INCLUDED

#include <windows.h>
#include "myconten.h"

namespace mylib
{

class CMyImage : public CMyContent
{
public:
	virtual ~CMyImage();
	virtual const SIZE& Size(HDC hdc);
	virtual int Draw(HDC hdc, int x, int y, PAINTSTRUCT& ps);

protected:
	CMyImage();

protected:
	BITMAPINFO* m_lpbmi;
	BYTE* m_lpvBits;
};

}	// namespace mylib

#endif	// MYIMAGE_H_INCLUDED
