// myconten.cpp

#include "myconten.h"

namespace mylib
{

CMyContent::CMyContent()
{
	m_sz.cx = 0;
	m_sz.cy = 0;
}

CMyContent::~CMyContent()
{
}

const SIZE&
CMyContent::Size(HDC hdc)
{
	return m_sz;
}

int
CMyContent::Draw(HDC hdc, int x, int y, PAINTSTRUCT& ps)
{
	return 0;
}

}	// namespace mylib
