// myconten.h

#ifndef MYCONTENT_H_INCLUDED
#define MYCONTENT_H_INCLUDED

#include <windows.h>

namespace mylib
{

class CMyContent
{
public:
	static const int TYPE_UNKNOWN = 0;
	static const int TEXT_PLAIN = 1;
	static const int IMAGE_BMP = 11;
	static const int IMAGE_PNG = 12;
	static const int IMAGE_JPEG = 13;
	static const int IMAGE_GIF = 14;

public:
	virtual ~CMyContent();
	virtual const SIZE& Size(HDC hdc);
	virtual int Draw(HDC hdc, int x, int y, PAINTSTRUCT& ps);

protected:
	CMyContent();

protected:
	SIZE m_sz;
};

}	// namespace mylib

#endif	// MYCONTENT_H_INCLUDED
