// mybmp.h
#ifndef MYBMP_H_INCLUDED
#define MYBMP_H_INCLUDED

#include "myimage.h"

namespace mylib
{

class CMyBMP : public CMyImage
{
public:
	CMyBMP(const TCHAR* szDocName);
	virtual ~CMyBMP();

protected:
	BYTE* m_lpFileData;
};

}	// namespace mylib

#endif	// MYBMP_H_INCLUDED
